@@ -281,6 +281,22 @@ class GuideElement(Element):
         self.current_parameters = None
 
         super().__init__(name, length=length, start_point=start_point, **kwargs)
+
+    def get_source_focus(self):
+        """
+        Returns dict describing how to focus on this guide opening
+
+        Should be overwritten in cases where it is not suitable to focus on
+        the module which inherits from this baseclass. For example a module
+        that want to focus on some point in the middle of the module, or a
+        gap which doesn't need focusing at all, in which case this method
+        can return None to tell the source to focus on the next part of the
+        guide that may have focus data.
+        """
+
+        return dict(width=self.start_width.name,
+                    height=self.start_height.name,
+                    dist=self.start_point_parameter.name)
     
     def setup_instrument_and_parameters(self, instrument, instrument_parameters):
         """
