@@ -136,6 +136,17 @@ class Kink(guide_elements.LosBreakerGuideElement):
         self.v_displacement = guide_elements.handle_input_parameter(name + "_v_displacement", v_displacement,
                                                                     default_min=-0.03, default_max=0.03)
 
+    def get_source_focus(self):
+        """
+        A source needs to focus at the end of a kink
+        """
+
+        effective_width = self.end_width.name + " + 2*" + self.h_displacement.name
+        effective_height = self.end_height.name + " + 2*" + self.v_displacement.name
+        end_distance = self.next_start_point_parameter.name
+
+        return dict(width=effective_width, height=effective_height, dist=end_distance)
+
     def los_input(self, value):
         """
         Input for breaking line of sight, single value input
