@@ -9,7 +9,7 @@ from guide_bot.logging.log_plotter_helpers import LogParameter
 from guide_bot.logging.log_plotter_helpers import GuideElementLog
 from guide_bot.logging.log_plotter_helpers import sort_permutation
 from guide_bot.logging.log_plotter_helpers import extract_parameters
-from guide_bot.base_elements.guide_elements import PositionAndRotation
+from guide_bot.base_elements.base_element_geometry import PositionAndRotation
 
 import guide_bot.elements.Element_gap as Gap
 import guide_bot.elements.Element_kink as Kink
@@ -18,8 +18,11 @@ import guide_bot.elements.Element_elliptic as Elliptic
 import guide_bot.elements.Element_curved as Curved
 import guide_bot.elements.Element_slit as Slit
 
+from guide_bot.elements.geometry_register import geometry_register
+
 from scipy.spatial.transform import Rotation as R
 
+
 class LogPlotter:
     def __init__(self, log_filename, guide_log_filename):
         if not os.path.isfile(log_filename):
@@ -49,6 +52,17 @@ class LogPlotter:
         self.data_fom_sorted = self.data[self.fom_sort_indices, :]
         self.fom_sorted = self.data_fom_sorted[:, 0]
 
+        self.category_dict = {}
+        for guide_element in self.guide_element_logs:
+            for simple_parameter_name in guide_element.parameter_type:
+                par_category = guide_element.parameter_type[simple_parameter_name]
+                par_name = guide_element.parameters[simple_parameter_name]
+                self.category_dict[par_name] = par_category
+
+        for par in self.parameters:
+            if par.name not in self.category_dict:
+                self.category_dict[par.name] = "global"
+
     def read_log_file(self):
         with open(self.filename) as file:
             file.readline()
@@ -120,71 +134,6 @@ class LogPlotter:
 
     def plot_guide_ax(self, ax, horizontal=True, data_line=None):
 
-        # If no data is given, plot best configuration
-        if data_line is None:
-            data_line = self.data_fom_sorted[-1, :]
-
-        # plot source
-        par_dict = extract_parameters(self.guide_element_logs[0], self.legend, data_line)
-        if horizontal:
-            if "start_width" in par_dict and "start_point" in par_dict:
-                start_point = par_dict["start_point"]
-                start_width = par_dict["start_width"]
-                ax.plot([start_point, start_point], [-0.5*start_width, 0.5*start_width], color="dimgrey")
-        else:
-            if "start_height" in par_dict and "start_point" in par_dict:
-                start_point = par_dict["start_point"]
-                start_height = par_dict["start_height"]
-                ax.plot([start_point, start_point], [-0.5*start_height, 0.5*start_height], color="dimgrey")
-
-        # plot sample
-        par_dict = extract_parameters(self.guide_element_logs[-1], self.legend, data_line)
-        if horizontal:
-            if "end_width" in par_dict and "next_start_point" in par_dict:
-                sample_width = par_dict["end_width"]
-                sample_position = par_dict["next_start_point"]
-                ax.plot([sample_position, sample_position], [-0.5 * sample_width, 0.5 * sample_width], color="dimgrey")
-        else:
-            if "end_height" in par_dict and "next_start_point" in par_dict:
-                sample_height = par_dict["end_height"]
-                sample_position = par_dict["next_start_point"]
-                ax.plot([sample_position, sample_position], [-0.5*sample_height, 0.5*sample_height], color="dimgrey")
-
-        color = "k"
-        for guide_element in self.guide_element_logs:
-
-            if color == "k":
-                color = "b"
-            elif color == "b":
-                color = "k"
-
-            par_dict = extract_parameters(guide_element, self.legend, data_line)
-            if guide_element.element_type == "Slit":
-                if horizontal:
-                    Slit.plot_element_horizontal(ax, par_dict, color=color)
-                else:
-                    Slit.plot_element_vertical(ax, par_dict, color=color)
-
-            if guide_element.element_type == "Straight":
-                if horizontal:
-                    Straight.plot_element_horizontal(ax, par_dict, color=color)
-                else:
-                    Straight.plot_element_vertical(ax, par_dict, color=color)
-
-            if guide_element.element_type == "Elliptic":
-                if horizontal:
-                    Elliptic.plot_element_horizontal(ax, par_dict, color=color)
-                else:
-                    Elliptic.plot_element_vertical(ax, par_dict, color=color)
-
-        ax.set_xlabel("Distance from source [m]")
-        if horizontal:
-            ax.set_ylabel("Horizontal [m]")
-        else:
-            ax.set_ylabel("Vertical [m]")
-
-    def plot_guide_center_line_ax(self, ax, horizontal=True, data_line=None):
-
         # If no data is given, plot best configuration
         if data_line is None:
             data_line = self.data_fom_sorted[-1, :]
@@ -207,6 +156,7 @@ class LogPlotter:
                 m_point, p_point = pos_and_rot.width_points(moderator_height)
                 ax.plot([m_point[1], p_point[1]], [m_point[0], p_point[0]], color="dimgrey")
 
+        # plot guide elements in alternating colors.
         color = "k"
         for guide_element in self.guide_element_logs:
 
@@ -215,49 +165,24 @@ class LogPlotter:
             elif color == "b":
                 color = "k"
 
+            # Get dict of generic parameters for this element type that contains the values from this data line
             par_dict = extract_parameters(guide_element, self.legend, data_line)
+            Geometry = geometry_register(guide_element.element_type)  # Get appropriate Geometry class for this type
+            geometry_object = Geometry(**par_dict)  # Initialize geometry object with the obtained pars / values
+            pos_and_rot = geometry_object.plot_on_ax(ax=ax, start_pr=pos_and_rot, horizontal=horizontal, color=color)
 
-            if guide_element.element_type == "Slit":
-                center_function = Slit.center_line
-
-            elif guide_element.element_type == "Gap":
-                center_function = Gap.center_line
-
-            elif guide_element.element_type == "Kink":
-                center_function = Kink.center_line
-
-            elif guide_element.element_type == "Straight":
-                center_function = Straight.center_line
-                dim_function = Straight.guide_dimensions
-                self.plot_element(ax, pos_and_rot, center_function, dim_function, par_dict, 2, horizontal, color)
-
-            elif guide_element.element_type == "Curved":
-                center_function = Curved.center_line
-                dim_function = Curved.guide_dimensions
-                self.plot_element(ax, pos_and_rot, center_function, dim_function, par_dict, 20, horizontal, color)
-
-            elif guide_element.element_type == "Elliptic":
-                center_function = Elliptic.center_line
-                dim_function = Elliptic.guide_dimensions
-                self.plot_element(ax, pos_and_rot, center_function, dim_function, par_dict, 30, horizontal, color)
-
-            else:
-                raise RuntimeError("Unknown type in plotter!")
-
-            pos_and_rot = center_function(pos_and_rot, par_dict, 1.0)
-
-        # plot sample
+        # plot target
         par_dict = extract_parameters(self.guide_element_logs[-1], self.legend, data_line)
         if horizontal:
             if "end_width" in par_dict:
-                sample_width = par_dict["end_width"]
-                m_point, p_point = pos_and_rot.width_points(sample_width)
+                target_width = par_dict["end_width"]
+                m_point, p_point = pos_and_rot.width_points(target_width)
                 ax.plot([m_point[2], p_point[2]], [m_point[0], p_point[0]], color="dimgrey")
 
         else:
             if "end_height" in par_dict:
-                sample_height = par_dict["end_height"]
-                m_point, p_point = pos_and_rot.height_points(sample_height)
+                target_height = par_dict["end_height"]
+                m_point, p_point = pos_and_rot.height_points(target_height)
                 ax.plot([m_point[2], p_point[2]], [m_point[1], p_point[1]], color="dimgrey")
 
         ax.set_xlabel("Distance from source [m]")
@@ -266,24 +191,6 @@ class LogPlotter:
         else:
             ax.set_ylabel("Vertical [m]")
 
-    def plot_element(self, ax, start_pr, center_function, dim_function, par_dict, n_points, horizontal, color):
-        distances = np.linspace(0, 1, n_points)
-        plus_array = np.zeros((n_points, 3))
-        minus_array = np.zeros((n_points, 3))
-
-        for index, distance in enumerate(distances):
-            this_pr = center_function(start_pr, par_dict, distance)
-            dim = dim_function(par_dict, distance, horizontal)
-            minus_array[index, :], plus_array[index, :] = this_pr.get_points(dim, horizontal)
-
-        if horizontal:
-            ax.plot(plus_array[:, 2], plus_array[:, 0], color=color)
-            ax.plot(minus_array[:, 2], minus_array[:, 0], color=color)
-        else:
-            ax.plot(plus_array[:, 2], plus_array[:, 1], color=color)
-            ax.plot(minus_array[:, 2], minus_array[:, 1], color=color)
-
-
     def plot_1D_fom_ax(self, ax, highlight_sorted_index=None, **kwargs):
         ax.scatter(range(len(self.fom)), self.fom, marker=".", label=self.scan_name, **kwargs)
         ax.plot(self.fom_sorted, color="orange")
@@ -299,6 +206,9 @@ class LogPlotter:
                        marker="x", label=self.scan_name, color='r', **kwargs)
 
     def plot_1D_cps_ax(self, ax, sorted=False, **kwargs):
+        """
+        Plotting change points
+        """
         n_guides = len(self.data[:, 0])
         pars_length_system_names = [par.name for par in self.parameters if par.owner == "length_system"]
         pars_length_system_indices = [self.legend.index(par) for par in pars_length_system_names]
@@ -318,6 +228,9 @@ class LogPlotter:
         ax.set_ylabel("start point [m]")
 
     def plot_1D_given_type_ax(self, ax, type="horizontal", sorted=False, only_free=True, **kwargs):
+        """
+        Plots parameters of given type
+        """
         found_par_names = set()
         for guide_element in self.guide_element_logs:
             for simple_parameter_name in guide_element.parameter_type:
@@ -327,7 +240,8 @@ class LogPlotter:
         if only_free:
             for parameter in self.parameters:
                 if parameter.name in found_par_names:
-                    if parameter.type not in ("FreeInstrumentParameter", "RelativeFreeInstrumentParameter"):
+                    free = ("FreeInstrumentParameter", "RelativeFreeInstrumentParameter", "LosInstrumentParameter")
+                    if parameter.type not in free:
                         found_par_names.remove(parameter.name)
 
         plotted_par_indices = [self.legend.index(par) for par in found_par_names if par in self.legend]
@@ -348,7 +262,105 @@ class LogPlotter:
         else:
             ax.set_xlabel("Optimizer iteration")
 
+    def plot_correlation_ax(self, ax, par1, par2, axis_labels=True, **kwargs):
+        par1_index = self.legend.index(par1)
+        par2_index = self.legend.index(par2)
+
+        # Todo truncate colormap to avoid completely transparent part
+        ax.scatter(self.data[:, par1_index], self.data[:, par2_index],
+                   #c=self.data[:, 0], cmap="Greens_r", marker=".", **kwargs)
+                   c=self.data[:, 0], cmap="jet", marker=".", **kwargs)
+
+        index_with_fom_zero = np.where(self.data[:, 0] == 0)
+        # index_with_fom_zero = np.where(data[:, 0] > 0.1*best_fom)
+        ax.scatter(self.data[index_with_fom_zero, par1_index], self.data[index_with_fom_zero, par2_index],
+                   marker="x", color="red", **kwargs)
+
+        if axis_labels:
+            ax.set_xlabel(par1)
+            ax.set_ylabel(par2)
+
+    def plot_correlation(self, par1, par2, **kwargs):
+
+        fig, ax = plt.subplots(figsize=(8, 6))
+        self.plot_correlation_ax(ax=ax, par1=par1, par2=par2, **kwargs)
+
+    def plot_all_correlations(self, figsize=(12,6), type=None, owner=None, category=None, **kwargs):
+
+        list_of_free_parameters = []
+        for par in self.parameters:
+            if par.limits == "" and par.type != "LosInstrumentParameter":
+                continue
+
+            if type is not None and par.type != type:
+                continue
+
+            if owner is not None and par.owner != owner:
+                continue
+
+            if category is not None and self.category_dict[par.name] != category:
+                continue
+
+            list_of_free_parameters.append(par.name)
+
+        n_pars = len(list_of_free_parameters)
+
+        if len(list_of_free_parameters) < 2:
+            print("Less than 2 parameters within search terms.")
+            return
+
+        fig, axs = plt.subplots(figsize=figsize, nrows=n_pars-1, ncols=n_pars-1)
+
+        if n_pars == 2:
+            par1 = list_of_free_parameters[0]
+            par2 = list_of_free_parameters[1]
+            self.plot_correlation_ax(axs, par1=par1, par2=par2, **kwargs)
+            return
+
+        for index1, par1 in enumerate(list_of_free_parameters):
+            for index2, par2 in enumerate(list_of_free_parameters):
+                used_index1 = index1
+                used_index2 = index2 - 1
+                if used_index1 == n_pars - 1 or used_index2 < 0:
+                    continue
+
+                if index1 >= index2:
+                    axs[used_index1, used_index2].set_axis_off()
+                else:
+                    axis_labels = used_index1 == used_index2
+                    # switch x and y axis by having par1=par2 and vice versa for nicer plotting
+                    self.plot_correlation_ax(axs[used_index1, used_index2], par1=par2, par2=par1,
+                                             axis_labels=axis_labels, **kwargs)
+                    
+                    if not axis_labels:
+                        axs[used_index1, used_index2].set_yticklabels([])
+                        axs[used_index1, used_index2].set_xticklabels([])
+
+    def print_parameter_types(self):
+        type_dict = {}
+        for par in self.parameters:
+            type_dict[par.type] = None
+
+        print(list(type_dict.keys()))
+
+    def print_parameter_owners(self):
+        owner_dict = {}
+        for par in self.parameters:
+            owner_dict[par.owner] = None
+
+        print(list(owner_dict.keys()))
+
+    def print_parameter_categories(self):
+        category_display = {}
+        for par in self.parameters:
+            category_display[self.category_dict[par.name]] = None
+
+        print(list(category_display))
+
     def plot_sim_duration_ax(self, ax, fom_correlation=False):
+        """
+        Plot simulation duration for each simulation iteration
+        """
 
         if "sim_start_t" not in self.legend or "sim_end_t" not in self.legend:
             print("Timing information not in log file")
@@ -383,6 +395,9 @@ class LogPlotter:
         self.plot_1D_given_type_ax(axs[3, 1], type="vertical", sorted=False)
 
     def plot_overview(self):
+        """
+        Plotting full overview
+        """
 
         fig = plt.figure(figsize=(13, 12), tight_layout=True)
         fig.suptitle(self.scan_name, fontsize=16)
@@ -393,10 +408,10 @@ class LogPlotter:
         self.plot_1D_cps_ax(fig.add_subplot(gs[0, 2]))
 
         geometry_h_ax = fig.add_subplot(gs[1, 0:2])
-        self.plot_guide_center_line_ax(geometry_h_ax, horizontal=True)
+        self.plot_guide_ax(geometry_h_ax, horizontal=True)
 
         geometry_v_ax = fig.add_subplot(gs[2, 0:2])
-        self.plot_guide_center_line_ax(geometry_v_ax, horizontal=False)
+        self.plot_guide_ax(geometry_v_ax, horizontal=False)
 
         par_h_ax = fig.add_subplot(gs[1, 2])
         self.plot_1D_given_type_ax(par_h_ax, type="horizontal")
