@@ -134,10 +134,12 @@ class PlotSum(BaseInterface):
                 locked_slider.disabled = True
 
     def unlock_all_sliders(self):
-        for key in self.target_scan_sliders:
-            self.target_scan_sliders[key].disabled = False
-        for key in self.moderator_scan_sliders:
-            self.moderator_scan_sliders[key].disabled = False
+        if self.target_scan_sliders is not None:
+            for key in self.target_scan_sliders:
+                self.target_scan_sliders[key].disabled = False
+        if self.moderator_scan_sliders is not None:
+            for key in self.moderator_scan_sliders:
+                self.moderator_scan_sliders[key].disabled = False
 
     def make_dropdown_scan_par(self):
 
