@@ -56,7 +56,7 @@ class MCPL_source(BaseSource):
         self.parameters.add("z_rotation", z_rotation)
 
 
-    def add_to_instrument(self, instrument, instrument_parameters, first_element):
+    def add_to_instrument(self, instrument, instrument_parameters, focus_info):
         """
         Add to instrument adds McStas code describing the source
 
@@ -72,8 +72,8 @@ class MCPL_source(BaseSource):
         instrument_parameters : InstrumentParameterContainer object
             Parameter container where parameters can be added for optimization
 
-        first_element : GuideElement
-            The first user provided GuideElement after the source
+        focus_info : dict
+            Dict with info on focusing to be used by the source
         """
 
         src = instrument.add_component("MCPL_source", "MCPL_input", after="Origin")
@@ -105,7 +105,7 @@ class MCPL_source(BaseSource):
         if self["scaling"] != 1.0:
             src.append_EXTEND(f"p *= {self['scaling']};")
 
-        face = instrument.add_component("moderator_face", "Shape")
+        face = instrument.add_component("moderator_face", "Shape", after=src)
         face.set_AT([0, 0, 0], RELATIVE="Origin")
         face.xwidth=self["width"]
         face.yheight=self["height"]
